==> libgtop-2.5.1.tar.bz2: libgtop-2.5.1/sysdeps/linux/proctime.c <==
/* $Id: proctime.c,v 1.25 2003/12/23 10:44:33 hadess Exp $ */

/* Copyright (C) 1998-99 Martin Baulig
   This file is part of LibGTop 1.0.

   Contributed by Martin Baulig <martin@home-of-linux.org>, April 1998.

   LibGTop is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.

   LibGTop is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with LibGTop; see the file COPYING. If not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <glibtop.h>
#include <glibtop/error.h>
#include <glibtop/proctime.h>

static const unsigned long _glibtop_sysdeps_proc_time =
(1L << GLIBTOP_PROC_TIME_UTIME) + (1L << GLIBTOP_PROC_TIME_CUTIME) +
(1L << GLIBTOP_PROC_TIME_STIME) + (1L << GLIBTOP_PROC_TIME_CSTIME) +
(1L << GLIBTOP_PROC_TIME_FREQUENCY) + (1L << GLIBTOP_PROC_TIME_TIMEOUT) +
(1L << GLIBTOP_PROC_TIME_IT_REAL_VALUE) + (1L << GLIBTOP_PROC_TIME_START_TIME);

static const unsigned long _glibtop_sysdeps_proc_time_smp =
(1L << GLIBTOP_PROC_TIME_XCPU_UTIME) + (1L << GLIBTOP_PROC_TIME_XCPU_STIME);

/* Init function. */

void
glibtop_init_proc_time_s (glibtop *server)
{
	server->sysdeps.proc_time = _glibtop_sysdeps_proc_time;

	if (server->ncpu)
		server->sysdeps.proc_time |= _glibtop_sysdeps_proc_time_smp;
}

/* Provides detailed information about a process. */

void
glibtop_get_proc_time_s (glibtop *server, glibtop_proc_time *buf, pid_t pid)
{
	char buffer [BUFSIZ], *p;
	int i;
	
	glibtop_init_s (&server, GLIBTOP_SYSDEPS_PROC_TIME, 0);

	memset (buf, 0, sizeof (glibtop_proc_time));

	if (proc_stat_to_buffer (buffer, pid))
		return;

	p = proc_stat_after_cmd (buffer);
	if (!p) return;

	p = skip_multiple_token (p, 11);

	buf->utime  = strtoull (p, &p, 0);
	buf->stime  = strtoull (p, &p, 0);
	buf->cutime = strtoull (p, &p, 0);
	buf->cstime = strtoull (p, &p, 0);

	p = skip_multiple_token (p, 2);

	buf->timeout       = strtoull (p, &p, 0);
	buf->it_real_value = strtoull (p, &p, 0);
	buf->start_time    = strtoull (p, &p, 0);

	buf->frequency = 100;

	buf->flags = _glibtop_sysdeps_proc_time;

	if (!server->ncpu)
		return;

	if (proc_file_to_buffer (buffer, "/proc/%d/cpu", pid))
		return;

	p = skip_token (buffer);
	buf->utime  = strtoull (p, &p, 0);
	buf->stime  = strtoull (p, &p, 0);

	for (i = 0; i < GLIBTOP_NCPU; i++) {
		if (strncmp (p+1, "cpu", 3) || !isdigit (p [4]))
			break;

		p += 6;
		buf->xcpu_utime [i] = strtoull (p, &p, 0);
		buf->xcpu_stime [i] = strtoull (p, &p, 0);
	}

	buf->flags |= _glibtop_sysdeps_proc_time_smp;
}

==> libgtop-2.7.90.tar.bz2: libgtop-2.7.90/sysdeps/linux/proctime.c <==
/* $Id: proctime.c,v 1.28 2004/08/03 11:40:37 bdejean Exp $ */

/* Copyright (C) 1998-99 Martin Baulig
   This file is part of LibGTop 1.0.

   Contributed by Martin Baulig <martin@home-of-linux.org>, April 1998.

   LibGTop is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.

   LibGTop is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with LibGTop; see the file COPYING. If not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <glibtop.h>
#include <glibtop/error.h>
#include <glibtop/proctime.h>
#include <glibtop/uptime.h>

static const unsigned long _glibtop_sysdeps_proc_time =
(1L << GLIBTOP_PROC_TIME_UTIME) + (1L << GLIBTOP_PROC_TIME_CUTIME) +
(1L << GLIBTOP_PROC_TIME_STIME) + (1L << GLIBTOP_PROC_TIME_CSTIME) +
(1L << GLIBTOP_PROC_TIME_FREQUENCY) + (1L << GLIBTOP_PROC_TIME_TIMEOUT) +
(1L << GLIBTOP_PROC_TIME_IT_REAL_VALUE) + (1L << GLIBTOP_PROC_TIME_START_TIME);

static const unsigned long _glibtop_sysdeps_proc_time_smp =
(1L << GLIBTOP_PROC_TIME_XCPU_UTIME) + (1L << GLIBTOP_PROC_TIME_XCPU_STIME);

/* Init function. */

void
glibtop_init_proc_time_s (glibtop *server)
{
	server->sysdeps.proc_time = _glibtop_sysdeps_proc_time;

	if (server->ncpu)
		server->sysdeps.proc_time |= _glibtop_sysdeps_proc_time_smp;
}

/* Provides detailed information about a process. */

void
glibtop_get_proc_time_s (glibtop *server, glibtop_proc_time *buf, pid_t pid)
{
	char buffer [BUFSIZ], *p;
	int i;

	glibtop_init_s (&server, GLIBTOP_SYSDEPS_PROC_TIME, 0);

	memset (buf, 0, sizeof (glibtop_proc_time));

	if (proc_stat_to_buffer (buffer, pid))
		return;

	p = proc_stat_after_cmd (buffer);
	if (!p) return;

	p = skip_multiple_token (p, 11);

	/* clock_t  (1/100 s) */
	buf->utime  = strtoull (p, &p, 0);
	buf->stime  = strtoull (p, &p, 0);
	buf->cutime = strtoull (p, &p, 0);
	buf->cstime = strtoull (p, &p, 0);

	p = skip_multiple_token (p, 3);

	// timeout is 0 on 2.4 and "thread_number" on 2.6
	// lets skip it (using previous skip_multiple_token)
	// buf->timeout       = strtoull (p, &p, 0);
	buf->it_real_value = strtoull (p, &p, 0);

	/* seconds since epoch */
	{
	  /* Linux provides start_time as clock_t representing
	     the start of <pid> after boot_time.
	     Let's use glibtop_get_uptime to get boot_time.
	     But i'm not sure if this is safe

	     See libgtop documentation.

	     #ifdef __KERNEL__
	     ...
	     *
	     * Have the 32 bit jiffies value wrap 5 minutes after boot
	     * so jiffies wrap bugs show up earlier.
	     *
	     #define INITIAL_JIFFIES ((unsigned long)(unsigned int) (-300*HZ))
	     ...
	     #endif

	     start_time may be incremented by INITIAL_JIFFIES, so start_time
	     may be not be exact. You may also get wrong start_time if your
	     system clock is not synchronised with you hardware clock.
	     'man hwclock'
	  */
	  glibtop_uptime up;
	  glibtop_get_uptime_s(server, &up);

	  buf->start_time = up.boot_time + strtoull (p, &p, 0) / 100;
	}

	buf->frequency = 100;

	buf->flags = _glibtop_sysdeps_proc_time;

	if (!server->ncpu)
		return;

	if (proc_file_to_buffer (buffer, "/proc/%d/cpu", pid))
		return;

	p = skip_token (buffer);
	buf->utime  = strtoull (p, &p, 0);
	buf->stime  = strtoull (p, &p, 0);

	for (i = 0; i < GLIBTOP_NCPU; i++) {
		if (strncmp (p+1, "cpu", 3) || !isdigit (p [4]))
			break;

		p += 6;
		buf->xcpu_utime [i] = strtoull (p, &p, 0);
		buf->xcpu_stime [i] = strtoull (p, &p, 0);
	}

	buf->flags |= _glibtop_sysdeps_proc_time_smp;
}
